#!/usr/bin/env perl

# This script automates the process of generating a set of evolved
# sequences using specific evolution parameters for seq-gen and
# sgEvolver

#
# In order to get this script to perform the desired simulation, edit the parameters
# below.  In particular, set the location of the ancestral donor sequence,
# the number of taxa (sequences), and the seqnames if you exceed 9 sequences.
# You will also need to edit the two for loops below to change which evolutionary
# events are varied and what range of scaling parameters is used to scale the
# branch lengths of the input phylogeny.
#
# This script will generate a directory for each simulation that contains a specification
# file describing the simulation to be done.  Use the 'simujobrun.pl' script to
# actually perform the simulation, alignment, and scoring of the alignment
#
# (c) 2004 Aaron Darling, All rights reserved
#

use strict;
use POSIX;
my $repetitions = 3; 

my $sequence_length = 1000000;
my $nt_sub_scale = 0.02;
my $gamma_shape = 1;
my $indel_rate = 0;
my $small_ht_rate = 0;
my $small_ht_size = 200;
my $large_ht_rate = 0;
my $large_ht_min = 10000;
my $large_ht_max = 60000;
my $inv_rate = 0;
my $inv_size = 50000;
my $nt_a_freq = .22;
my $nt_c_freq = .28;
my $nt_g_freq = .28;
my $nt_t_freq = .22;

my $tree_filename = "test.tree";
#my $ancestral_donor = "/home/koadman/sequence/diarrhea/o157sakai.raw";
my $ancestral_donor = "/u/d/a/darling/mauve/o157sakai.raw";
my $ancestral_seq_name = "ancestral.dat";
my $donor_seq_name = "donor.dat";
my $seqgen_out_name = "seqgen.dat";
my $evolved_seqs_name = "evolved.dat";	# file for the fully evolved alignment output
my $evolved_seqs_fname = "evolved_seqs.fas";	# file for the ungapped sequences
my @seqnames = ("Taxon1","Taxon2","Taxon3","Taxon4","Taxon5","Taxon6","Taxon7","Taxon8","Taxon9");
my $seq_count = 2;
my $seqgen_random = int(rand(0x7FFFFFFF));
my $seqgen_donor_random = int(rand(0x7FFFFFFF));
my $sgEvolver_random = int(rand(0x7FFFFFFF));

my $evolution_params_file = "evolution_params.txt";
my $skip_runs = 0;	# the first data set to generate
my $last_run = -1;	# the last data set to generate, set to negative for a run to completion

#generate job descriptions that can be used to perform simulation, alignment, and scoring

my $total_runs = 0;
my $x_increment = .05;
my $y_increment = .0002;
my $x_count = 0;
my $y_count = 0;
for( my $repI = 0; $repI < $repetitions; $repI++ ){
	$x_count = 0;
	for( my $x_val = 0.0000000001; $x_val < 1; $x_val += $x_increment ){
# modulate nt substitution rate on x-axis
		$nt_sub_scale = $x_val;
		$x_count++;
		$y_count = 0;
		for( my $y_val = 0; $y_val < .004; $y_val += $y_increment ){
			$y_count++;
# modulate inv_rate on y-axis
			$inv_rate = $y_val;
			if( $total_runs < $skip_runs ){
				$total_runs++;
				next;
			}
			next if( $last_run >= 0 && $total_runs > $last_run );

		# make a directory for the job files
			my $job_dir = "alignjob.$total_runs";
			mkdir( $job_dir );
			chdir( $job_dir );
			$total_runs++;

		#stat the ancestral donor to get its file size
			my @stat_info = stat($ancestral_donor);
			my $size_modulus = $stat_info[7] - $sequence_length - $sequence_length;
			my $ancestral_start = POSIX::floor( rand( $size_modulus ) );
			my $donor_start = $ancestral_start + $sequence_length + POSIX::floor( rand($stat_info[7] - $ancestral_start - $sequence_length - $sequence_length) );

		# write alignment params to a file
			open( PARAMS, ">$evolution_params_file"  );
			print PARAMS "seq_length=$sequence_length\n";
			print PARAMS "nt_sub_rate=$nt_sub_scale\n";
			print PARAMS "gamma_shape=$gamma_shape\n";
			print PARAMS "indel_rate=$indel_rate\n";
			print PARAMS "small_ht_rate=$small_ht_rate\n";
			print PARAMS "small_ht_size=$small_ht_size\n";
			print PARAMS "large_ht_rate=$large_ht_rate\n";
			print PARAMS "large_ht_min=$large_ht_min\n";
			print PARAMS "large_ht_max=$large_ht_max\n";
			print PARAMS "inv_rate=$inv_rate\n";
			print PARAMS "inv_size=$inv_size\n";
			print PARAMS "nt_a_freq=$nt_a_freq\n";
			print PARAMS "nt_c_freq=$nt_c_freq\n";
			print PARAMS "nt_g_freq=$nt_g_freq\n";
			print PARAMS "nt_t_freq=$nt_t_freq\n";

			print PARAMS "tree_filename=$tree_filename\n";
			print PARAMS "ancestral_donor=$ancestral_donor\n";
			print PARAMS "ancestral_seq_name=$ancestral_seq_name\n";
			print PARAMS "donor_seq_name=$donor_seq_name\n";
			print PARAMS "seqgen_out_name=$seqgen_out_name\n";
			print PARAMS "evolved_seqs_name=$evolved_seqs_name\n";
			print PARAMS "evolved_seqs_fname=$evolved_seqs_fname\n";

			print PARAMS "ancestral_start=$ancestral_start\n";
			print PARAMS "donor_start=$donor_start\n";

			print PARAMS "seqgen_random=$seqgen_random\n";
			print PARAMS "seqgen_donor_random=$seqgen_donor_random\n";
			print PARAMS "sgEvolver_random=$sgEvolver_random\n";

			close PARAMS;
		
		# generate a tree file
			open( TREEOUT, ">$tree_filename" );
			print TREEOUT "($seqnames[0]:.5,$seqnames[1]:.5);";
#			print TREEOUT "(((($seqnames[0]:0.12979,($seqnames[1]:0.06944,$seqnames[2]:0.06473)".
#							":0.09078):0.00963,($seqnames[3]:0.09309,$seqnames[4]:0.09051)".
#							":0.07744):0.03698,$seqnames[5]:0.19844):0.2646,($seqnames[6]".
#							":0.11243,($seqnames[7]:0.06886,$seqnames[8]:0.06788):0.04328):0.35968);";

			close TREEOUT;
			print "Run: $total_runs\n";
		# leave the directory
			chdir( ".." );
		}
		# kludge for seq-gen's inability to accept a 0 mutation rate	
		if( $x_val < $x_increment ){
			$x_val = 0;
		}
	}
}

my $condor_job_file = "mauveAlign.condor";
open( JOB_FILE, ">$condor_job_file" );
print JOB_FILE qq{
####################                   
# Condor job submission for mauveAligner
####################                                                   

universe       = vanilla
Executable     = simujobrun.pl
arguments      = $evolution_params_file
Requirements   = Memory >= 128 && OpSys == "LINUX" && Arch =="INTEL"
Rank = Memory >= 256
Image_Size     = 900 Meg                                                

should_transfer_files = YES
when_to_transfer_output = ON_EXIT
transfer_input_files = $tree_filename, $evolution_params_file
notification = Never
coresize = 1000
copy_to_spool = False

Error   = simujobrun.err
Output  = simujobrun.out
initialdir = alignjob.\$(Process)
Log     = simujobrun.log

Queue $total_runs
};

close JOB_FILE;

my $run_description_file = "run_description.txt";
open( RUN_FILE, ">$run_description_file" );
# print out the number of cols, rows, and the increments used for each
print RUN_FILE "$x_count\t$y_count\t$x_increment\t$y_increment\n";
close RUN_FILE;


